
/* 
 * Copyright (C) 2000-2004 the xine project
 * Copyright (C) 2004-2005 Maximilian Schwerin
 * 
 * This file is part of oxine a free media player.
 * The code in this file was copied from the xine-ui project.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: lang.c 3789 2009-04-25 20:06:26Z mschwerin $
 *
 */
#include "config.h"

#include "string.h"
#include "stdlib.h"
#include "utils.h"
#include "i18n.h"
#include "lang.h"

/*
 * Our LC_MESSAGES handling.
 */

/*
  locale -a tell us (* mean we already handle):
  -----------------
  *  C
  *  POSIX
     af_ZA
     ar_AE
     ar_BH
     ar_DZ
     ar_EG
     ar_IN
     ar_IQ
     ar_JO
     ar_KW
     ar_LB
     ar_LY
     ar_MA
     ar_OM
     ar_QA
     ar_SA
     ar_SD
     ar_SY
     ar_TN
     ar_YE
     be_BY
     bg_BG
     bokmal
     bokmål
     ca_ES
     ca_ES@euro
     catalan
     croatian
  *  cs_CZ
  *  cs_CZ.UTF8
     czech
     da_DK
     danish
     dansk
     de_AT
     de_AT@euro
     de_BE
     de_BE@euro
     de_CH
  *  de_DE
  *  de_DE@euro
     de_LU
     de_LU@euro
  *  deutsch
     dutch
     eesti
     el_GR
     en_AU
     en_BW
     en_CA
     en_DK
  *  en_GB
     en_HK
     en_IE
     en_IE@euro
     en_IN
     en_NZ
     en_PH
     en_SG
     en_US
     en_ZA
     en_ZW
     es_AR
     es_BO
     es_CL
     es_CO
     es_CR
     es_DO
     es_EC
  *  es_ES
  *  es_ES@euro
     es_GT
     es_HN
     es_MX
     es_NI
     es_PA
     es_PE
     es_PR
     es_PY
     es_SV
     es_US
     es_UY
     es_VE
     estonian
     et_EE
     eu_ES
     eu_ES@euro
     fa_IR
     fi_FI
     fi_FI@euro
     finnish
     fo_FO
     fr_BE
     fr_BE@euro
     fr_CA
     fr_CH
  *  fr_FR
  *  fr_FR@euro
     fr_LU
     fr_LU@euro
  *  français
  *  french
     ga_IE
     ga_IE@euro
     galego
     galician
  *  german
     gl_ES
     gl_ES@euro
     greek
     gv_GB
     he_IL
     hebrew
     hi_IN
     hr_HR
     hrvatski
     hu_HU
     hungarian
     icelandic
     id_ID
     is_IS
     it_CH
     it_IT
     it_IT@euro
     italian
     iw_IL
     ja_JP
     ja_JP.eucjp
     ja_JP.ujis
     japanese
     japanese.euc
     japanese.sjis
     kl_GL
     ko_KR
     ko_KR.euckr
     ko_KR.UTF-8
     korean
     korean.euc
     kw_GB
     lithuanian
     lt_LT
     lv_LV
     mk_MK
     mr_IN
     ms_MY
     mt_MT
     nb_NO
     nb_NO.ISO-8859-1
     nl_BE
     nl_BE@euro
     nl_NL
     nl_NL@euro
     nn_NO
     no_NO
     norwegian
     nynorsk
 *   pl_PL
 *   pl_PL.UTF-8
     polish
     portuguese
  *  pt_BR
     pt_PT
     pt_PT@euro
     ro_RO
     romanian
     ru_RU
     ru_RU.koi8r
     ru_UA
     russian
     sk_SK
     sl_SI
     slovak
     slovene
     slovenian
  *  spanish
     sq_AL
     sr_YU
     sr_YU@cyrillic
     sv_FI
     sv_FI@euro
     sv_SE
     swedish
     ta_IN
     te_IN
     th_TH
     thai
     tr_TR
     turkish
  *  uk_UA
     vi_VN
     zh_CN
     zh_CN.gb18030
     zh_CN.gbk
     zh_HK
     zh_TW
     zh_TW.euctw

*/

/* Global definition */
static langs_t _langs[] = {
    /* English section */
    {"C", ENGLISH, "en", NULL},                                 /* WARNING: This should be the first entry */
    {"POSIX", ENGLISH, "en", NULL},
    {"en_GB", ENGLISH, "en", "ISO-8859-1"},

    /* French section */
    {"fr_FR", FRENCH, "fr", "ISO-8859-1"},
    {"fr_FR@euro", FRENCH, "fr", "ISO-8859-1"},
    {"français", FRENCH, "fr", "ISO-8859-1"},
    {"french", FRENCH, "fr", "ISO-8859-1"},

    /* German section */
    {"de_DE", GERMAN, "de", "ISO-8859-1"},
    {"de_DE@euro", GERMAN, "de", "ISO-8859-1"},
    {"deutsch", GERMAN, "de", "ISO-8859-1"},
    {"german", GERMAN, "de", "ISO-8859-1"},

    /* Portuguese section */
    {"pt_BR", PORTUGUESE, "pt_BR", "ISO-8859-1"},

    /* Spanish section */
    {"es_ES", SPANISH, "es", "ISO-8859-1"},
    {"es_ES@euro", SPANISH, "es", "ISO-8859-1"},
    {"spanish", SPANISH, "es", "ISO-8859-1"},

    /* Polish */
    {"pl_PL", POLISH, "pl", "ISO-8859-2"},
    {"pl_PL.UTF-8", POLISH, "pl", "ISO-8859-2"},

    /* Ukrainian */
    {"uk_UA", UKRAINIAN, "uk", "KOI8-U"},
    {"uk_UA.UTF-8", UKRAINIAN, "uk", "KOI8-U"},

    /* Czech */
    {"cs_CZ", CZECH, "cs", "ISO-8859-2"},
    {"cs_CZ.UTF-8", CZECH, "cs", "ISO-8859-2"},

    /* NORWEGIAN */
    {"nb_NO", NORWEGIAN_BOKMAL, "nb", "ISO-8859-1"},
    {"nb_NO.ISO-8859-1", NORWEGIAN_BOKMAL, "nb", "ISO-8859-1"},

    /* The ultimate solution */
    {NULL, ENGLISH, "en", "ISO-8859-1"}
};

/* ISO 639-1 */
typedef struct {

    /**
     * Three letter (ISO 639-2) language code (bibliographic).
     */
    char *iso_639_2_bibl;

    /**
     * Three letter (ISO 639-2) language code (terminology).
     */
    char *iso_639_2_term;

    /**
     * Two letter (ISO 639-1) language code.
     */
    char *iso_639_1;

    /**
     * Language name.
     */
    char *language;
} iso639_t;

/*
 * Return language from iso639-1 two letters.
 */
const char *
get_language_from_iso639_1 (char *lang_code)
{
    iso639_t iso639[] = {
        {"aar", "   ", "aa", _("Afar")},
        {"abk", "   ", "ab", _("Abkhazian")},
        {"ace", "   ", "  ", _("Achinese")},
        {"ach", "   ", "  ", _("Acoli")},
        {"ada", "   ", "  ", _("Adangme")},
        {"ady", "   ", "  ", _("Adyghe")},
        {"afa", "   ", "  ", _("Afro-Asiatic (Other)")},
        {"afh", "   ", "  ", _("Afrihili")},
        {"afr", "   ", "af", _("Afrikaans")},
        {"ain", "   ", "  ", _("Ainu")},
        {"aka", "   ", "ak", _("Akan")},
        {"akk", "   ", "  ", _("Akkadian")},
        {"alb", "sqi", "sq", _("Albanian")},
        {"ale", "   ", "  ", _("Aleut")},
        {"alg", "   ", "  ", _("Algonquian languages")},
        {"alt", "   ", "  ", _("Southern Altai")},
        {"amh", "   ", "am", _("Amharic")},
        {"ang", "   ", "  ", _("English, Old (ca.450-1100)")},
        {"anp", "   ", "  ", _("Angika")},
        {"apa", "   ", "  ", _("Apache languages")},
        {"ara", "   ", "ar", _("Arabic")},
        {"arc", "   ", "  ", _("Aramaic")},
        {"arg", "   ", "an", _("Aragonese")},
        {"arm", "hye", "hy", _("Armenian")},
        {"arn", "   ", "  ", _("Mapudungun")},
        {"arp", "   ", "  ", _("Arapaho")},
        {"arw", "   ", "  ", _("Arawak")},
        {"asm", "   ", "as", _("Assamese")},
        {"ast", "   ", "  ", _("Asturian")},
        {"ath", "   ", "  ", _("Athapascan languages")},
        {"aus", "   ", "  ", _("Australian languages")},
        {"ava", "   ", "av", _("Avaric")},
        {"ave", "   ", "ae", _("Avestan")},
        {"awa", "   ", "  ", _("Awadhi")},
        {"aym", "   ", "ay", _("Aymara")},
        {"aze", "   ", "az", _("Azerbaijani")},
        {"bad", "   ", "  ", _("Banda languages")},
        {"bai", "   ", "  ", _("Bamileke languages")},
        {"bak", "   ", "ba", _("Bashkir")},
        {"bal", "   ", "  ", _("Baluchi")},
        {"bam", "   ", "bm", _("Bambara")},
        {"ban", "   ", "  ", _("Balinese")},
        {"baq", "eus", "eu", _("Basque")},
        {"bas", "   ", "  ", _("Basa")},
        {"bat", "   ", "  ", _("Baltic (Other)")},
        {"bej", "   ", "  ", _("Beja")},
        {"bel", "   ", "be", _("Belarusian")},
        {"bem", "   ", "  ", _("Bemba")},
        {"ben", "   ", "bn", _("Bengali")},
        {"ber", "   ", "  ", _("Berber (Other)")},
        {"bho", "   ", "  ", _("Bhojpuri")},
        {"bih", "   ", "bh", _("Bihari")},
        {"bik", "   ", "  ", _("Bikol")},
        {"bin", "   ", "  ", _("Bini")},
        {"bis", "   ", "bi", _("Bislama")},
        {"bla", "   ", "  ", _("Siksika")},
        {"bnt", "   ", "  ", _("Bantu (Other)")},
        {"bos", "   ", "bs", _("Bosnian")},
        {"bra", "   ", "  ", _("Braj")},
        {"bre", "   ", "br", _("Breton")},
        {"btk", "   ", "  ", _("Batak languages")},
        {"bua", "   ", "  ", _("Buriat")},
        {"bug", "   ", "  ", _("Buginese")},
        {"bul", "   ", "bg", _("Bulgarian")},
        {"bur", "mya", "my", _("Burmese")},
        {"byn", "   ", "  ", _("Blin")},
        {"cad", "   ", "  ", _("Caddo")},
        {"cai", "   ", "  ", _("Central American Indian (Other)")},
        {"car", "   ", "  ", _("Galibi Carib")},
        {"cat", "   ", "ca", _("Catalan")},
        {"cau", "   ", "  ", _("Caucasian (Other)")},
        {"ceb", "   ", "  ", _("Cebuano")},
        {"cel", "   ", "  ", _("Celtic (Other)")},
        {"cha", "   ", "ch", _("Chamorro")},
        {"chb", "   ", "  ", _("Chibcha")},
        {"che", "   ", "ce", _("Chechen")},
        {"chg", "   ", "  ", _("Chagatai")},
        {"chi", "zho", "zh", _("Chinese")},
        {"chk", "   ", "  ", _("Chuukese")},
        {"chm", "   ", "  ", _("Mari")},
        {"chn", "   ", "  ", _("Chinook jargon")},
        {"cho", "   ", "  ", _("Choctaw")},
        {"chp", "   ", "  ", _("Chipewyan")},
        {"chr", "   ", "  ", _("Cherokee")},
        {"chu", "   ", "cu", _("Church Slavic")},
        {"chv", "   ", "cv", _("Chuvash")},
        {"chy", "   ", "  ", _("Cheyenne")},
        {"cmc", "   ", "  ", _("Chamic languages")},
        {"cop", "   ", "  ", _("Coptic")},
        {"cor", "   ", "kw", _("Cornish")},
        {"cos", "   ", "co", _("Corsican")},
        {"cpe", "   ", "  ", _("Creoles and pidgins, English based (Other)")},
        {"cpf", "   ", "  ", _("Creoles and pidgins, French-based (Other)")},
        {"cpp", "   ", "  ", _("Creoles and pidgins, Portuguese-based (Other)")},
        {"cre", "   ", "cr", _("Cree")},
        {"crh", "   ", "  ", _("Crimean Tatar")},
        {"crp", "   ", "  ", _("Creoles and pidgins (Other)")},
        {"csb", "   ", "  ", _("Kashubian")},
        {"cus", "   ", "  ", _("Cushitic (Other)")},
        {"cze", "ces", "cs", _("Czech")},
        {"dak", "   ", "  ", _("Dakota")},
        {"dan", "   ", "da", _("Danish")},
        {"dar", "   ", "  ", _("Dargwa")},
        {"day", "   ", "  ", _("Land Dayak languages")},
        {"del", "   ", "  ", _("Delaware")},
        {"den", "   ", "  ", _("Slave (Athapascan)")},
        {"dgr", "   ", "  ", _("Dogrib")},
        {"din", "   ", "  ", _("Dinka")},
        {"div", "   ", "dv", _("Divehi")},
        {"doi", "   ", "  ", _("Dogri")},
        {"dra", "   ", "  ", _("Dravidian (Other)")},
        {"dsb", "   ", "  ", _("Lower Sorbian")},
        {"dua", "   ", "  ", _("Duala")},
        {"dum", "   ", "  ", _("Dutch, Middle (ca.1050-1350)")},
        {"dut", "nld", "nl", _("Dutch")},
        {"dyu", "   ", "  ", _("Dyula")},
        {"dzo", "   ", "dz", _("Dzongkha")},
        {"efi", "   ", "  ", _("Efik")},
        {"egy", "   ", "  ", _("Egyptian (Ancient)")},
        {"eka", "   ", "  ", _("Ekajuk")},
        {"elx", "   ", "  ", _("Elamite")},
        {"eng", "   ", "en", _("English")},
        {"enm", "   ", "  ", _("English, Middle (1100-1500)")},
        {"epo", "   ", "eo", _("Esperanto")},
        {"est", "   ", "et", _("Estonian")},
        {"ewe", "   ", "ee", _("Ewe")},
        {"ewo", "   ", "  ", _("Ewondo")},
        {"fan", "   ", "  ", _("Fang")},
        {"fao", "   ", "fo", _("Faroese")},
        {"fat", "   ", "  ", _("Fanti")},
        {"fij", "   ", "fj", _("Fijian")},
        {"fil", "   ", "  ", _("Filipino")},
        {"fin", "   ", "fi", _("Finnish")},
        {"fiu", "   ", "  ", _("Finno-Ugrian (Other)")},
        {"fon", "   ", "  ", _("Fon")},
        {"fre", "fra", "fr", _("French")},
        {"frm", "   ", "  ", _("French, Middle (ca.1400-1600)")},
        {"fro", "   ", "  ", _("French, Old (842-ca.1400)")},
        {"frr", "   ", "  ", _("Northern Frisian")},
        {"frs", "   ", "  ", _("Eastern Frisian")},
        {"fry", "   ", "fy", _("Western Frisian")},
        {"ful", "   ", "ff", _("Fulah")},
        {"fur", "   ", "  ", _("Friulian")},
        {"gaa", "   ", "  ", _("Ga")},
        {"gay", "   ", "  ", _("Gayo")},
        {"gba", "   ", "  ", _("Gbaya")},
        {"gem", "   ", "  ", _("Germanic (Other)")},
        {"geo", "kat", "ka", _("Georgian")},
        {"ger", "deu", "de", _("German")},
        {"gez", "   ", "  ", _("Geez")},
        {"gil", "   ", "  ", _("Gilbertese")},
        {"gla", "   ", "gd", _("Gaelic")},
        {"gle", "   ", "ga", _("Irish")},
        {"glg", "   ", "gl", _("Galician")},
        {"glv", "   ", "gv", _("Manx")},
        {"gmh", "   ", "  ", _("German, Middle High (ca.1050-1500)")},
        {"goh", "   ", "  ", _("German, Old High (ca.750-1050)")},
        {"gon", "   ", "  ", _("Gondi")},
        {"gor", "   ", "  ", _("Gorontalo")},
        {"got", "   ", "  ", _("Gothic")},
        {"grb", "   ", "  ", _("Grebo")},
        {"grc", "   ", "  ", _("Greek, Ancient (to 1453)")},
        {"gre", "ell", "el", _("Greek, Modern (1453-)")},
        {"grn", "   ", "gn", _("Guarani")},
        {"gsw", "   ", "  ", _("Swiss German")},
        {"guj", "   ", "gu", _("Gujarati")},
        {"gwi", "   ", "  ", _("Gwich'in")},
        {"hai", "   ", "  ", _("Haida")},
        {"hat", "   ", "ht", _("Haitian")},
        {"hau", "   ", "ha", _("Hausa")},
        {"haw", "   ", "  ", _("Hawaiian")},
        {"heb", "   ", "he", _("Hebrew")},
        {"her", "   ", "hz", _("Herero")},
        {"hil", "   ", "  ", _("Hiligaynon")},
        {"him", "   ", "  ", _("Himachali")},
        {"hin", "   ", "hi", _("Hindi")},
        {"hit", "   ", "  ", _("Hittite")},
        {"hmn", "   ", "  ", _("Hmong")},
        {"hmo", "   ", "ho", _("Hiri Motu")},
        {"hsb", "   ", "  ", _("Upper Sorbian")},
        {"hun", "   ", "hu", _("Hungarian")},
        {"hup", "   ", "  ", _("Hupa")},
        {"iba", "   ", "  ", _("Iban")},
        {"ibo", "   ", "ig", _("Igbo")},
        {"ice", "isl", "is", _("Icelandic")},
        {"ido", "   ", "io", _("Ido")},
        {"iii", "   ", "ii", _("Sichuan Yi")},
        {"ijo", "   ", "  ", _("Ijo languages")},
        {"iku", "   ", "iu", _("Inuktitut")},
        {"ile", "   ", "ie", _("Interlingue")},
        {"ilo", "   ", "  ", _("Iloko")},
        {"inc", "   ", "  ", _("Indic (Other)")},
        {"ind", "   ", "id", _("Indonesian")},
        {"ine", "   ", "  ", _("Indo-European (Other)")},
        {"inh", "   ", "  ", _("Ingush")},
        {"ipk", "   ", "ik", _("Inupiaq")},
        {"ira", "   ", "  ", _("Iranian (Other)")},
        {"iro", "   ", "  ", _("Iroquoian languages")},
        {"ita", "   ", "it", _("Italian")},
        {"jav", "   ", "jv", _("Javanese")},
        {"jbo", "   ", "  ", _("Lojban")},
        {"jpn", "   ", "ja", _("Japanese")},
        {"jpr", "   ", "  ", _("Judeo-Persian")},
        {"jrb", "   ", "  ", _("Judeo-Arabic")},
        {"kaa", "   ", "  ", _("Kara-Kalpak")},
        {"kab", "   ", "  ", _("Kabyle")},
        {"kac", "   ", "  ", _("Kachin")},
        {"kal", "   ", "kl", _("Kalaallisut")},
        {"kam", "   ", "  ", _("Kamba")},
        {"kan", "   ", "kn", _("Kannada")},
        {"kar", "   ", "  ", _("Karen languages")},
        {"kas", "   ", "ks", _("Kashmiri")},
        {"kau", "   ", "kr", _("Kanuri")},
        {"kaw", "   ", "  ", _("Kawi")},
        {"kaz", "   ", "kk", _("Kazakh")},
        {"kbd", "   ", "  ", _("Kabardian")},
        {"kha", "   ", "  ", _("Khasi")},
        {"khi", "   ", "  ", _("Khoisan (Other)")},
        {"khm", "   ", "km", _("Central Khmer")},
        {"kho", "   ", "  ", _("Khotanese")},
        {"kik", "   ", "ki", _("Kikuyu")},
        {"kin", "   ", "rw", _("Kinyarwanda")},
        {"kir", "   ", "ky", _("Kirghiz")},
        {"kmb", "   ", "  ", _("Kimbundu")},
        {"kok", "   ", "  ", _("Konkani")},
        {"kom", "   ", "kv", _("Komi")},
        {"kon", "   ", "kg", _("Kongo")},
        {"kor", "   ", "ko", _("Korean")},
        {"kos", "   ", "  ", _("Kosraean")},
        {"kpe", "   ", "  ", _("Kpelle")},
        {"krc", "   ", "  ", _("Karachay-Balkar")},
        {"krl", "   ", "  ", _("Karelian")},
        {"kro", "   ", "  ", _("Kru languages")},
        {"kru", "   ", "  ", _("Kurukh")},
        {"kua", "   ", "kj", _("Kuanyama")},
        {"kum", "   ", "  ", _("Kumyk")},
        {"kur", "   ", "ku", _("Kurdish")},
        {"kut", "   ", "  ", _("Kutenai")},
        {"lad", "   ", "  ", _("Ladino")},
        {"lah", "   ", "  ", _("Lahnda")},
        {"lam", "   ", "  ", _("Lamba")},
        {"lao", "   ", "lo", _("Lao")},
        {"lat", "   ", "la", _("Latin")},
        {"lav", "   ", "lv", _("Latvian")},
        {"lez", "   ", "  ", _("Lezghian")},
        {"lim", "   ", "li", _("Limburgan")},
        {"lin", "   ", "ln", _("Lingala")},
        {"lit", "   ", "lt", _("Lithuanian")},
        {"lol", "   ", "  ", _("Mongo")},
        {"loz", "   ", "  ", _("Lozi")},
        {"ltz", "   ", "lb", _("Luxembourgish")},
        {"lua", "   ", "  ", _("Luba-Lulua")},
        {"lub", "   ", "lu", _("Luba-Katanga")},
        {"lug", "   ", "lg", _("Ganda")},
        {"lui", "   ", "  ", _("Luiseno")},
        {"lun", "   ", "  ", _("Lunda")},
        {"luo", "   ", "  ", _("Luo (Kenya and Tanzania)")},
        {"lus", "   ", "  ", _("Lushai")},
        {"mac", "mkd", "mk", _("Macedonian")},
        {"mad", "   ", "  ", _("Madurese")},
        {"mag", "   ", "  ", _("Magahi")},
        {"mah", "   ", "mh", _("Marshallese")},
        {"mai", "   ", "  ", _("Maithili")},
        {"mak", "   ", "  ", _("Makasar")},
        {"mal", "   ", "ml", _("Malayalam")},
        {"man", "   ", "  ", _("Mandingo")},
        {"mao", "mri", "mi", _("Maori")},
        {"map", "   ", "  ", _("Austronesian (Other)")},
        {"mar", "   ", "mr", _("Marathi")},
        {"mas", "   ", "  ", _("Masai")},
        {"may", "msa", "ms", _("Malay")},
        {"mdf", "   ", "  ", _("Moksha")},
        {"mdr", "   ", "  ", _("Mandar")},
        {"men", "   ", "  ", _("Mende")},
        {"mga", "   ", "  ", _("Irish, Middle (900-1200)")},
        {"mic", "   ", "  ", _("Mi'kmaq")},
        {"min", "   ", "  ", _("Minangkabau")},
        {"mkh", "   ", "  ", _("Mon-Khmer (Other)")},
        {"mlg", "   ", "mg", _("Malagasy")},
        {"mlt", "   ", "mt", _("Maltese")},
        {"mnc", "   ", "  ", _("Manchu")},
        {"mni", "   ", "  ", _("Manipuri")},
        {"mno", "   ", "  ", _("Manobo languages")},
        {"moh", "   ", "  ", _("Mohawk")},
        {"mol", "   ", "mo", _("Moldavian")},
        {"mon", "   ", "mn", _("Mongolian")},
        {"mos", "   ", "  ", _("Mossi")},
        {"mun", "   ", "  ", _("Munda languages")},
        {"mus", "   ", "  ", _("Creek")},
        {"mwl", "   ", "  ", _("Mirandese")},
        {"mwr", "   ", "  ", _("Marwari")},
        {"myn", "   ", "  ", _("Mayan languages")},
        {"myv", "   ", "  ", _("Erzya")},
        {"nah", "   ", "  ", _("Nahuatl languages")},
        {"nai", "   ", "  ", _("North American Indian")},
        {"nap", "   ", "  ", _("Neapolitan")},
        {"nau", "   ", "na", _("Nauru")},
        {"nav", "   ", "nv", _("Navajo")},
        {"nbl", "   ", "nr", _("Ndebele, South")},
        {"nde", "   ", "nd", _("Ndebele, North")},
        {"ndo", "   ", "ng", _("Ndonga")},
        {"nds", "   ", "  ", _("Low German")},
        {"nep", "   ", "ne", _("Nepali")},
        {"new", "   ", "  ", _("Nepal Bhasa")},
        {"nia", "   ", "  ", _("Nias")},
        {"nic", "   ", "  ", _("Niger-Kordofanian (Other)")},
        {"niu", "   ", "  ", _("Niuean")},
        {"nno", "   ", "nn", _("Norwegian Nynorsk")},
        {"nob", "   ", "nb", _("Bokmål, Norwegian")},
        {"nog", "   ", "  ", _("Nogai")},
        {"non", "   ", "  ", _("Norse, Old")},
        {"nor", "   ", "no", _("Norwegian")},
        {"nso", "   ", "  ", _("Pedi")},
        {"nub", "   ", "  ", _("Nubian languages")},
        {"nwc", "   ", "  ", _("Classical Newari")},
        {"nya", "   ", "ny", _("Chichewa")},
        {"nym", "   ", "  ", _("Nyamwezi")},
        {"nyn", "   ", "  ", _("Nyankole")},
        {"nyo", "   ", "  ", _("Nyoro")},
        {"nzi", "   ", "  ", _("Nzima")},
        {"oci", "   ", "oc", _("Occitan (post 1500)")},
        {"oji", "   ", "oj", _("Ojibwa")},
        {"ori", "   ", "or", _("Oriya")},
        {"orm", "   ", "om", _("Oromo")},
        {"osa", "   ", "  ", _("Osage")},
        {"oss", "   ", "os", _("Ossetian")},
        {"ota", "   ", "  ", _("Turkish, Ottoman (1500-1928)")},
        {"oto", "   ", "  ", _("Otomian languages")},
        {"paa", "   ", "  ", _("Papuan (Other)")},
        {"pag", "   ", "  ", _("Pangasinan")},
        {"pal", "   ", "  ", _("Pahlavi")},
        {"pam", "   ", "  ", _("Pampanga")},
        {"pan", "   ", "pa", _("Panjabi")},
        {"pap", "   ", "  ", _("Papiamento")},
        {"pau", "   ", "  ", _("Palauan")},
        {"peo", "   ", "  ", _("Persian, Old (ca.600-400 B.C.)")},
        {"per", "fas", "fa", _("Persian")},
        {"phi", "   ", "  ", _("Philippine (Other)")},
        {"phn", "   ", "  ", _("Phoenician")},
        {"pli", "   ", "pi", _("Pali")},
        {"pol", "   ", "pl", _("Polish")},
        {"pon", "   ", "  ", _("Pohnpeian")},
        {"por", "   ", "pt", _("Portuguese")},
        {"pra", "   ", "  ", _("Prakrit languages")},
        {"pro", "   ", "  ", _("Provençal, Old (to 1500)")},
        {"pus", "   ", "ps", _("Pushto")},
        {"que", "   ", "qu", _("Quechua")},
        {"raj", "   ", "  ", _("Rajasthani")},
        {"rap", "   ", "  ", _("Rapanui")},
        {"rar", "   ", "  ", _("Rarotongan")},
        {"roa", "   ", "  ", _("Romance (Other)")},
        {"roh", "   ", "rm", _("Romansh")},
        {"rom", "   ", "  ", _("Romany")},
        {"rum", "ron", "ro", _("Romanian")},
        {"run", "   ", "rn", _("Rundi")},
        {"rup", "   ", "  ", _("Aromanian")},
        {"rus", "   ", "ru", _("Russian")},
        {"sad", "   ", "  ", _("Sandawe")},
        {"sag", "   ", "sg", _("Sango")},
        {"sah", "   ", "  ", _("Yakut")},
        {"sai", "   ", "  ", _("South American Indian (Other)")},
        {"sal", "   ", "  ", _("Salishan languages")},
        {"sam", "   ", "  ", _("Samaritan Aramaic")},
        {"san", "   ", "sa", _("Sanskrit")},
        {"sas", "   ", "  ", _("Sasak")},
        {"sat", "   ", "  ", _("Santali")},
        {"scc", "srp", "sr", _("Serbian")},
        {"scn", "   ", "  ", _("Sicilian")},
        {"sco", "   ", "  ", _("Scots")},
        {"scr", "hrv", "hr", _("Croatian")},
        {"sel", "   ", "  ", _("Selkup")},
        {"sem", "   ", "  ", _("Semitic (Other)")},
        {"sga", "   ", "  ", _("Irish, Old (to 900)")},
        {"shn", "   ", "  ", _("Shan")},
        {"sid", "   ", "  ", _("Sidamo")},
        {"sin", "   ", "si", _("Sinhala")},
        {"sio", "   ", "  ", _("Siouan languages")},
        {"sit", "   ", "  ", _("Sino-Tibetan (Other)")},
        {"sla", "   ", "  ", _("Slavic (Other)")},
        {"slo", "slk", "sk", _("Slovak")},
        {"slv", "   ", "sl", _("Slovenian")},
        {"sma", "   ", "  ", _("Southern Sami")},
        {"sme", "   ", "se", _("Northern Sami")},
        {"smi", "   ", "  ", _("Sami languages (Other)")},
        {"smj", "   ", "  ", _("Lule Sami")},
        {"smn", "   ", "  ", _("Inari Sami")},
        {"smo", "   ", "sm", _("Samoan")},
        {"sms", "   ", "  ", _("Skolt Sami")},
        {"sna", "   ", "sn", _("Shona")},
        {"snd", "   ", "sd", _("Sindhi")},
        {"snk", "   ", "  ", _("Soninke")},
        {"sog", "   ", "  ", _("Sogdian")},
        {"som", "   ", "so", _("Somali")},
        {"son", "   ", "  ", _("Songhai languages")},
        {"sot", "   ", "st", _("Sotho, Southern")},
        {"spa", "   ", "es", _("Spanish")},
        {"srd", "   ", "sc", _("Sardinian")},
        {"srn", "   ", "  ", _("Sranan Tongo")},
        {"srr", "   ", "  ", _("Serer")},
        {"ssa", "   ", "  ", _("Nilo-Saharan (Other)")},
        {"ssw", "   ", "ss", _("Swati")},
        {"suk", "   ", "  ", _("Sukuma")},
        {"sun", "   ", "su", _("Sundanese")},
        {"sus", "   ", "  ", _("Susu")},
        {"sux", "   ", "  ", _("Sumerian")},
        {"swa", "   ", "sw", _("Swahili")},
        {"swe", "   ", "sv", _("Swedish")},
        {"syr", "   ", "  ", _("Syriac")},
        {"tah", "   ", "ty", _("Tahitian")},
        {"tai", "   ", "  ", _("Tai (Other)")},
        {"tam", "   ", "ta", _("Tamil")},
        {"tat", "   ", "tt", _("Tatar")},
        {"tel", "   ", "te", _("Telugu")},
        {"tem", "   ", "  ", _("Timne")},
        {"ter", "   ", "  ", _("Tereno")},
        {"tet", "   ", "  ", _("Tetum")},
        {"tgk", "   ", "tg", _("Tajik")},
        {"tgl", "   ", "tl", _("Tagalog")},
        {"tha", "   ", "th", _("Thai")},
        {"tib", "bod", "bo", _("Tibetan")},
        {"tig", "   ", "  ", _("Tigre")},
        {"tir", "   ", "ti", _("Tigrinya")},
        {"tiv", "   ", "  ", _("Tiv")},
        {"tkl", "   ", "  ", _("Tokelau")},
        {"tlh", "   ", "  ", _("Klingon")},
        {"tli", "   ", "  ", _("Tlingit")},
        {"tmh", "   ", "  ", _("Tamashek")},
        {"tog", "   ", "  ", _("Tonga (Nyasa)")},
        {"ton", "   ", "to", _("Tonga (Tonga Islands)")},
        {"tpi", "   ", "  ", _("Tok Pisin")},
        {"tsi", "   ", "  ", _("Tsimshian")},
        {"tsn", "   ", "tn", _("Tswana")},
        {"tso", "   ", "ts", _("Tsonga")},
        {"tuk", "   ", "tk", _("Turkmen")},
        {"tum", "   ", "  ", _("Tumbuka")},
        {"tup", "   ", "  ", _("Tupi languages")},
        {"tur", "   ", "tr", _("Turkish")},
        {"tut", "   ", "  ", _("Altaic (Other)")},
        {"tvl", "   ", "  ", _("Tuvalu")},
        {"twi", "   ", "tw", _("Twi")},
        {"tyv", "   ", "  ", _("Tuvinian")},
        {"udm", "   ", "  ", _("Udmurt")},
        {"uga", "   ", "  ", _("Ugaritic")},
        {"uig", "   ", "ug", _("Uighur")},
        {"ukr", "   ", "uk", _("Ukrainian")},
        {"umb", "   ", "  ", _("Umbundu")},
        {"urd", "   ", "ur", _("Urdu")},
        {"uzb", "   ", "uz", _("Uzbek")},
        {"vai", "   ", "  ", _("Vai")},
        {"ven", "   ", "ve", _("Venda")},
        {"vie", "   ", "vi", _("Vietnamese")},
        {"vol", "   ", "vo", _("Volapük")},
        {"vot", "   ", "  ", _("Votic")},
        {"wak", "   ", "  ", _("Wakashan languages")},
        {"wal", "   ", "  ", _("Walamo")},
        {"war", "   ", "  ", _("Waray")},
        {"was", "   ", "  ", _("Washo")},
        {"wel", "cym", "cy", _("Welsh")},
        {"wen", "   ", "  ", _("Sorbian languages")},
        {"wln", "   ", "wa", _("Walloon")},
        {"wol", "   ", "wo", _("Wolof")},
        {"xal", "   ", "  ", _("Kalmyk")},
        {"xho", "   ", "xh", _("Xhosa")},
        {"yao", "   ", "  ", _("Yao")},
        {"yap", "   ", "  ", _("Yapese")},
        {"yid", "   ", "yi", _("Yiddish")},
        {"yor", "   ", "yo", _("Yoruba")},
        {"ypk", "   ", "  ", _("Yupik languages")},
        {"zap", "   ", "  ", _("Zapotec")},
        {"zen", "   ", "  ", _("Zenaga")},
        {"zha", "   ", "za", _("Zhuang")},
        {"znd", "   ", "  ", _("Zande languages")},
        {"zul", "   ", "zu", _("Zulu")},
        {"zun", "   ", "  ", _("Zuni")},
        {"nqo", "   ", "  ", _("N'Ko")},
        {"zza", "   ", "  ", _("Zaza")},
        {NULL, NULL, NULL}
    };

    if (lang_code) {
        char *tl = lang_code;

        while ((*tl == ' ') && (*tl != '\0')) {
            tl++;
        }

        if (tl && (strlen (tl) == 2)) {
            int i;

            for (i = 0; iso639[i].iso_639_1; i++) {
                if (str_equals (iso639[i].iso_639_1, tl)) {
                    return iso639[i].language;
                }
            }
        }

        if (tl && (strlen (tl) == 3)) {
            int i;

            for (i = 0; iso639[i].iso_639_2_bibl; i++) {
                if (str_equals (iso639[i].iso_639_2_bibl, tl)) {
                    return iso639[i].language;
                }
                if (str_equals (iso639[i].iso_639_2_term, tl)) {
                    return iso639[i].language;
                }
            }
        }

        return lang_code;
    }
    else {
        return _("Unknown");
    }
}

/*
 * Return a langs_t pointer on static struct langs[];
 */
const langs_t *
get_lang (void)
{
    langs_t *l;
    char *lcmsg = setlocale (LC_MESSAGES, NULL);

    if (lcmsg) {
        for (l = _langs; l->lang != NULL; l++) {
            if (strc_starts_with (l->lang, lcmsg)) {
                return l;
            }
        }
    }

    return &_langs[0];
}

const langs_t *
get_default_lang (void)
{
    return &_langs[0];
}


void untranslated (void);

void
untranslated (void)
{
    /* These entries are needed so the titles used in the standard
     * mainmenu.xml file appear in the translation files. */
    _("Listen to Music");
    _("Watch Films");
    _("View Images");
    _("Watch Television");
    _("Play a DVD");
    _("Play a Video CD");
    _("Play an Audio CD");
    _("Currently playing");
    _("Playback settings");
    _("CD / DVD Extractor");
    _("Edit Settings");
    _("Show Tools");
    _("Show Help");
    _("Shutdown");

    /* These entries are needed so that genres are appear in the translation
     * files. */
    _("Blues");
    _("Classic Rock");
    _("Country");
    _("Dance");
    _("Disco");
    _("Funk");
    _("Grunge");
    _("Hip-Hop");
    _("Jazz");
    _("Metal");
    _("New Age");
    _("Oldies");
    _("Other");
    _("Pop");
    _("R&B");
    _("Rap");
    _("Reggae");
    _("Rock");
    _("Techno");
    _("Industrial");
    _("Alternative");
    _("Ska");
    _("Death Metal");
    _("Pranks");
    _("Soundtrack");
    _("Euro-Techno");
    _("Ambient");
    _("Trip-Hop");
    _("Vocal");
    _("Jazz+Funk");
    _("Fusion");
    _("Trance");
    _("Classical");
    _("Instrumental");
    _("Acid");
    _("House");
    _("Game");
    _("Sound Clip");
    _("Gospel");
    _("Noise");
    _("AlternRock");
    _("Bass");
    _("Soul");
    _("Punk");
    _("Space");
    _("Meditative");
    _("Instrumental Pop");
    _("Instrumental Rock");
    _("Ethnic");
    _("Gothic");
    _("Darkwave");
    _("Techno-Industrial");
    _("Electronic");
    _("Pop-Folk");
    _("Eurodance");
    _("Dream");
    _("Southern Rock");
    _("Comedy");
    _("Cult");
    _("Gangsta");
    _("Top 40");
    _("Christian Rap");
    _("Pop/Funk");
    _("Jungle");
    _("Native American");
    _("Cabaret");
    _("New Wave");
    _("Psychadelic");
    _("Rave");
    _("Showtunes");
    _("Trailer");
    _("Lo-Fi");
    _("Tribal");
    _("Acid Punk");
    _("Acid Jazz");
    _("Polka");
    _("Retro");
    _("Musical");
    _("Rock & Roll");
    _("Hard Rock");
    _("Folk");
    _("Folk-Rock");
    _("National Folk");
    _("Swing");
    _("Fast Fusion");
    _("Bebob");
    _("Latino");
    _("Revival");
    _("Celtic");
    _("Bluegrass");
    _("Avantgarde");
    _("Gothic Rock");
    _("Progressive Rock");
    _("Psychedelic Rock");
    _("Symphonic Rock");
    _("Slow Rock");
    _("Big Band");
    _("Chorus");
    _("Easy Listening");
    _("Acoustic");
    _("Humour");
    _("Speech");
    _("Chanson");
    _("Opera");
    _("Chamber Music");
    _("Sonata");
    _("Symphony");
    _("Booty Bass");
    _("Primus");
    _("Porn Groove");
    _("Satire");
    _("Slow Jam");
    _("Club");
    _("Tango");
    _("Samba");
    _("Folklore");
    _("Ballad");
    _("Power Ballad");
    _("Rhythmic Soul");
    _("Freestyle");
    _("Duet");
    _("Punk Rock");
    _("Drum Solo");
    _("A capella");
    _("Euro-House");
    _("Dance Hall");
}
