
/*
 * Copyright (C) 2004-2005 Maximilian Schwerin
 *
 * This file is part of oxine a free media player.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: logger.h 3618 2008-12-12 14:13:12Z mschwerin $
 *
 */

#ifndef HAVE_LOGGER_H
#define HAVE_LOGGER_H

typedef enum {
    LEVEL_FATAL = 0,
    LEVEL_ERROR = 1,
    LEVEL_WARN = 2,
    LEVEL_INFO = 3,
    LEVEL_DEBUG = 4,
    LEVEL_TODO = 5,
    LEVEL_ENTER = 6,
} logger_level_t;

void logger_init (const char *filename);
void logger_free (void);

void _log (logger_level_t level, const char *func, const char *file,
           int line, const char *format, ...);

#ifdef DEBUG
#define debug(...)      _log(LEVEL_DEBUG,   NULL,       __FILE__, __LINE__, __VA_ARGS__)
#define enter(...)      _log(LEVEL_ENTER,   __func__,   __FILE__, __LINE__, __VA_ARGS__)
#else
#define debug(...)
#define enter(...)
#endif

#define log(level,...)  _log(level,         NULL,       __FILE__, __LINE__, __VA_ARGS__)
#define todo(...)       _log(LEVEL_TODO,    NULL,       __FILE__, __LINE__, __VA_ARGS__)
#define info(...)       _log(LEVEL_INFO,    NULL,       __FILE__, __LINE__, __VA_ARGS__)
#define warn(...)       _log(LEVEL_WARN,    NULL,       __FILE__, __LINE__, __VA_ARGS__)
#define error(...)      _log(LEVEL_ERROR,   NULL,       __FILE__, __LINE__, __VA_ARGS__)
#define fatal(...)      _log(LEVEL_FATAL,   NULL,       __FILE__, __LINE__, __VA_ARGS__)

#endif /* HAVE_LOGGER_H */
