
/*
 * Copyright (C) 2009 Maximilian Schwerin
 *
 * This file is part of oxine a free media player.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: menu_fileinfo.c 3899 2009-05-05 17:43:58Z mschwerin $
 *
 */

#include "config.h"

#include "heap.h"
#include "filelist.h"
#include "list.h"
#include "logger.h"
#include "oxine.h"
#include "playlist.h"
#include "utils.h"

#include "menu_base.h"
#include "menu_fileinfo.h"


extern oxine_t *oxine;

static char *title = NULL;
static l_list_t *info_list = NULL;

static otk_widget_t *fileinfo_menu_window = NULL;
static otk_widget_t *fileinfo_menu_info = NULL;
static otk_widget_t *fileinfo_menu_title = NULL;

static void fileinfo_menu_show (void *p);

static void
fileinfo_free (void *p)
{
    ho_free (p);
}


static void
fileinfo_menu_event_handler (void *p, oxine_event_t * event)
{
    if (!is_current_menu (fileinfo_menu_show)) {
        return;
    }
    switch (event->type) {
    case OXINE_EVENT_KEY:
        switch (event->source.key) {
        case OXINE_KEY_BLUE:
            event->source.key = OXINE_KEY_BACK;
            odk_oxine_event_send (oxine->odk, event);
            event->source.key = OXINE_KEY_NULL;
            break;
        default:
            break;
        }
        break;
    default:
        break;
    }
}


static void
fileinfo_menu_show_info (int x, int y, int w, int h)
{
    int x1 = x;
    int y1 = y;
    int w1 = w;
    int h1 = h;

#if 0
#ifdef HAVE_OSD_IMAGE
    int x3 = x;
    int y3 = y;
    int w3 = h;
    int h3 = h;
    otk_border_new (oxine->otk, x3, y3, w3, h3);

    int a2 = (ODK_ALIGN_CENTER | ODK_ALIGN_VCENTER);
    int w2 = h - 10;
    int h2 = h - 10;
    int x2 = x + 5 + (w2 / 2);
    int y2 = y + 5 + (h2 / 2);

    filelist_menu_image = otk_image_new (oxine->otk, x2, y2,
                                         w2, h2, NULL, a2, false);

    x1 += w3 + 10;
    w1 -= w3 + 10;
#endif /* HAVE_OSD_IMAGE */
#endif

    fileinfo_menu_info = otk_infolist_new (oxine->otk, x1, y1,
                                           w1, h1, 26, 26);
}


static void
fileinfo_menu_show (void *p)
{
    char *infoitem = NULL;

    if (fileinfo_menu_window) {
        otk_set_current_window (oxine->otk, fileinfo_menu_window);
        otk_list_clear (fileinfo_menu_info);
        goto out_show;
    }

    fileinfo_menu_window = create_new_window (true, true);

    int x = 20;
    int y = 100;
    int w = odk_osd_get_width (oxine->odk) - 40;
    int h = odk_osd_get_height (oxine->odk) - 120;

    int x0 = x;
    int y0 = y;
    int w0 = w;
    int a0 = (OTK_ALIGN_LEFT | OTK_ALIGN_VCENTER);
    fileinfo_menu_title =
        otk_label_new (oxine->otk, x0, y0, w0, 35, a0, title);
    otk_widget_set_fontsize (fileinfo_menu_title, 32);

    int x1 = x;
    int y1 = y + 40;
    int w1 = w;
    int h1 = h - 40;
    fileinfo_menu_show_info (x1, y1, w1, h1);

  out_show:
    infoitem = (char *) l_list_first (info_list);
    while (infoitem) {
        otk_infoitem_new (fileinfo_menu_info, infoitem);
        infoitem = l_list_next (info_list, infoitem);
    }

    otk_label_set_text (fileinfo_menu_title, title);
    set_current_menu (fileinfo_menu_show, NULL);

    show_user_interface (NULL);
    show_menu_background (NULL);
}


static void
fileinfo_menu_enter (l_list_t * info_list_original,
                     const char *title_original)
{
    odk_add_event_handler (oxine->odk, fileinfo_menu_event_handler,
                           oxine, EVENT_HANDLER_PRIORITY_NORMAL);

    if (!info_list) {
        info_list = l_list_new ();
    }
    else {
        l_list_clear (info_list, fileinfo_free);
    }

    void *p = l_list_first (info_list_original);
    while (p) {
        l_list_append (info_list, ho_strdup (p));
        p = l_list_next (info_list_original, p);
    }

    ho_free (title);
    title = ho_strdup (title_original);

    fileinfo_menu_show (NULL);
}


void
show_menu_fileitem_info (fileitem_t * fileitem)
{
    char *title = ho_strdup_printf ("%s - %s", fileitem->title,
                                    fileitem->parent_list->title_complete);
    fileinfo_menu_enter (fileitem_get_info_list (fileitem), title);
    ho_free (title);
}


void
show_menu_playitem_info (playitem_t * playitem)
{
    fileinfo_menu_enter (playitem_get_info_list (playitem), playitem->title);
}


void
free_menu_fileinfo (void)
{
    ho_free (title);

    if (info_list) {
        l_list_free (info_list, fileinfo_free);
        info_list = NULL;
    }
}
