
/*
 * Copyright (C) 2008 Maximilian Schwerin
 *
 * This file is part of oxine a free media player.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: utils_covers.h 3468 2008-09-04 13:17:01Z mschwerin $
 *
 */

#ifndef HAVE_UTILS_COVERS_H
#define HAVE_UTILS_COVERS_H

#include "types.h"

/**
 * Returns the MRL of a cover image for the specified artist and album or the
 * specified ASIN. The user must always provide an artist and an album and may
 * provide a valid ASIN. The returned string must be freed. The returned MRL
 * points to a file on a local filesystem.
 *
 * @param artist                The name of the artist.
 * @param album                 The name of the album.
 * @param asin                  The Amazon Store Identification Number.
 *
 * @return                      The MRL of the cover image or NULL if no image
 *                              could be found.
 */
char *get_cover_image_from_amazon (const char *artist, const char *album, const char *asin);

/**
 * Returns the MRL of a cover image for the specified MRL. The returned string
 * must be freed.
 *
 * @param mrl                   An MRL of a directory or media file for which
 *                              to retrieve the cover image.
 *
 * @return                      The MRL of the cover image or NULL if no image
 *                              could be found.
 */
char *get_cover_image_from_mrl (const char *mrl);


/**
 * Returns TRUE if the the MRL points to a non-local file or to a local file
 * that exists.
 */
bool is_cover_image_valid (const char *mrl);


#endif /* HAVE_UTILS_COVERS_H */

