
/*
 * Copyright (C) 2004-2005 Maximilian Schwerin
 *
 * This file is part of oxine a free media player.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: utils_gui.h 3913 2009-05-14 11:44:54Z mschwerin $
 *
 */

#ifndef HAVE_UTILS_GUI_H
#define HAVE_UTILS_GUI_H

#include "i18n.h"
#include "otk.h"
#include "playlist.h"

/**
 * Schedules the GUI to be hidden after the given timeout.
 *
 * @param                       The time in ms to wait before hiding the GUI.
 */
void schedule_hide_gui_job (int timeout);

/**
 * Cancels the job to hide the GUI. If no job is active this does nothing.
 */
void cancel_hide_gui_job (void);


/**
 * Schedules an event to be executed after the timeout expires.
 *
 * @param timeout               The time in ms to wait before sending the
 *                              event to the event handler.
 * @param event                 The event to send. The event data must be
 *                              allocated on the stack and will be freed
 *                              after sending. If scheduling of the job
 *                              fails the event data is freed.
 *  
 * @return                      The ID of the scheduled job or -1 if an
 *                              error occured. 
 */
int schedule_event_job (int timeout, oxine_event_t * ev);


/**
 * Schedules an event to be executed after the timeout expires.
 *
 * @param timeout               The time in ms to wait before sending the
 *                              event to the event handler.
 * @param id                    The ID of the event to send.
 *  
 * @return                      The ID of the scheduled job or -1 if an
 *                              error occured. 
 */
int schedule_simple_event_job (int timeout, oxine_event_id_t id);


/**
 * Cancels the job with the given ID. The event data is automatically
 * freed.
 */
void cancel_event_job (int * job);


void show_user_interface (void *data_p);

void hide_user_interface (void *data_p);

void show_menu_background (const char *background);

void show_download_progress_info (void *p);

otk_widget_t *create_new_window (bool persistent, bool with_clock);

bool show_stream_parameter (odk_stream_param_t param);

#endif /* HAVE_UTILS_GUI_H */
