
/*
 * Copyright (C) 2008 Maximilian Schwerin
 *
 * This file is part of oxine a free media player.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: utils_io.c 3550 2008-11-01 12:47:49Z mschwerin $
 *
 */

#include "config.h"

#include <errno.h>
#include <unistd.h>

#include "utils.h"


size_t
readline (int fd, char * buffer, size_t length)
{
    char b;
    int pos = 0;
    int size = 0;

    do {
        size = read (fd, &b, 1);

        if (size == 0) {
            /* do nothing */
        }
        /* signal interrupted read(), try again. */
        else if ((size == -1) && (errno == EINTR)) {
            size = 1;
        }
        /* other error, we quit with an error code. */
        else if ((size == -1) && (errno != EINTR)) {
            return -1;
        }
        /* got a byte, add to buffer if space left. */
        else if ((size == +1) && (pos < (length - 1))) {
            buffer[pos++] = b;
        }
    } while ((b != '\r') && (b != '\n') && (size > 0));

    buffer[pos] = '\0';

    return pos;
}
