
/*
 * Copyright (C) 2008 Maximilian Schwerin
 *
 * This file is part of oxine a free media player.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: utils_network.c 3201 2008-03-02 10:46:55Z mschwerin $
 *
 */
#include "config.h"

#include <errno.h>
#include <pthread.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>

#include "logger.h"
#include "utils.h"

#define BUFFER_SIZE             2048

int
server_init (int server_port, int queue_length)
{
    int ret;
    int listen_socket = 0;
    struct sockaddr_in server_addr;

    /* Create socket */
    listen_socket = socket (AF_INET, SOCK_STREAM, 0);
    if (listen_socket == -1) {
        error (_("Failed to create socket: %s!"), strerror (errno));
        return -1;
    }

    /* Socket-Option einstellen: IP/Port-Adresse
     * darf sofort wieder vergeben werden */
    int optval = 1;
    ret = setsockopt (listen_socket, SOL_SOCKET, SO_REUSEADDR,
                      (void *) &optval, sizeof (optval));
    if (ret == -1) {
        error (_("Failed to set socket options: %s!"), strerror (errno));
        return -1;
    }

    /* Socket mit "Namen", d.h. mit Port verbinden */
    memset ((void *) &server_addr, 0, sizeof (server_addr));
    server_addr.sin_family = AF_INET;
    server_addr.sin_addr.s_addr = htonl (INADDR_ANY);
    server_addr.sin_port = htons (server_port);
    ret = bind (listen_socket, (struct sockaddr *) &server_addr,
                sizeof (server_addr));
    if (ret == -1) {
        error (_("Failed to bind port to socket: %s!"), strerror (errno));
        return -1;
    }

    /* Socket fuer eingehende Verbindungen aktivieren */
    ret = listen (listen_socket, queue_length);
    if (ret == -1) {
        error (_("Failed to listen on socket: %s!"), strerror (errno));
        return -1;
    }

    return listen_socket;
}


int
server_printf (int client_socket, const char *format, ...)
{
    int res = 0;
    va_list args;
    char buffer[BUFFER_SIZE];

    va_start (args, format);
    vsnprintf (buffer, BUFFER_SIZE, format, args);
    va_end (args);

    if ((res = write (client_socket, buffer, strlen (buffer))) == -1) {
        error (_("Failed to write to socket: %s!"), strerror (errno));
    }
    else {
        debug ("> %s", trim_whitespace (buffer));
    }

    return res;
}
