
/*
 * Copyright (C) 2006 Maximilian Schwerin
 *
 * This file is part of oxine a free media player.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * $Id: vdr.h 3823 2009-04-28 20:00:16Z mschwerin $
 *
 */

#ifndef HAVE_VDR_H
#define HAVE_VDR_H

#include "list.h"
#include "types.h"
#include "filelist.h"

#ifdef HAVE_VDR


bool vdr_append_disc_stat (l_list_t *info_list);


/**
 * Start VDR playback.
 */
bool vdr_play (void);


/**
 * Creates a timer from an EPG entry.
 */
bool vdr_timer_create (fileitem_t *fileitem);


/**
 * Reads the currently available a VDR channels in to a filelist.
 */
bool vdr_channels_read (filelist_t * filelist);


/**
 * Plays a VDR channel.
 */
bool vdr_channel_play (fileitem_t * fileitem);


/**
 * Reads the currently available VDR recordings into a filelist.
 */
bool vdr_recordings_read (filelist_t * filelist);


/**
 * Plays a VDR recording.
 */
bool vdr_recording_play (fileitem_t * fileitem);


/**
 * Removes a VDR recording.
 */
bool vdr_recording_remove (fileitem_t * fileitem);


/**
 * Reads the currently programmed VDR timers into a filelist.
 */
bool vdr_timers_read (filelist_t * filelist);


/**
 * Get info about a VDR timer.
 */
bool vdr_timer_read_info (fileitem_t * fileitem);


/**
 * Remove a VDR timer.
 */
bool vdr_timer_remove (fileitem_t * fileitem);


/**
 * Activates or deactivates a timer.
 */
bool vdr_timer_set_active (fileitem_t * fileitem, bool activate);


/**
 * Returns TRUE if VDR is recording at the moment.
 */
bool vdr_is_recording (void);


/**
 * Registers any configuration keys used by this module.
 */
void vdr_register_config (void);


/**
 * Frees static data of VDR module.
 */
void vdr_static_free (void);


#endif /* HAVE_VDR */
#endif /* HAVE_VDR_H */
